
% REPRODUCING EXPERIMENT 1

clear; % we need to clear the variables in workspace before learning. 

% CONSTANTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
scale = .7;  % scale of Gabors, length = 17 pixels
norient = 15;  % number of orientations
epsilon = .1;  % allowed correlation between selected Gabors
T = 0.;        % threshold for responses
Totalsketch = 60;  % allowed number of selected Gabors   
Upperbound = 16.; % saturation value
sub = 2;    % subsampling rate for Gabor
Lrange = 3; % allowed shifting in location (times sub pixels)
Orange = 1; % allowed shifting in orientation
%%% Note: Lrange*sub is the pixels allowed to shift along normal
%%%       Lrange*sub should be the same in learning and testing

LIK = 1.; SHUTUP = 1.;  % use log-likelihood score. If LIK=0, then use active correlation
% SHUTUP is to neglect the positions with low average responses
                       
% LIK = 0; SHUTUP = 1.;   


% TRAINING EXAMPLES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
posFolder = 'cars';  % image input folder
Iname = dir([posFolder '\*.jpg']);
nimage = size(Iname, 1);
Isize = zeros(nimage, 2);
for (i = 1 : nimage)
    I{i} = imresize(double(rgb2gray(imread([posFolder '\' Iname(i).name]))), 1);
    Ishow{i} = imread([posFolder '\' Iname(i).name]); 
    Isize(i, :) = size(I{i});
end
sx = min(Isize(:, 1)); sy = min(Isize(:, 2)); 
for (i = 1 : nimage)
    I{i} = I{i}(1:sx, 1:sy); 
end   % make the sizes of the training images the same

% FILTERING IMAGES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[allfilter, allsymbol] = makefilter(scale, norient);  % generate Gabor filters 
h = (size(allfilter{1}, 1)-1)/2;  % half size of Gabor
C = corr(allfilter, epsilon);    % generate the inhibition maps
allfiltered = applyfilter(I, allfilter, T);

alldr = [30, 35, 5, 29, 22, 3, 37]; 
ndr = length(alldr); 

stay = 30; 
dr = cell(ndr+2, Totalsketch+stay);
for (i1 = 1:ndr+2)
  for (i2 = 1:Totalsketch+stay)
     dr{i1, i2} = zeros(sx, sy); 
  end
end


% VARIABLES FOR MEX-C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Mi = zeros(1, Totalsketch);  % orientation of selected Gabor
Mx = zeros(1, Totalsketch);  % position of selected Gabor
My = zeros(1, Totalsketch); 
Mm = zeros(1, Totalsketch);  % average response of selected Gabor
sym = zeros(size(I{1}));  % symbolic plot of selected Gabor
Asym = cell(1, nimage);   % symbolic plot for each image with active Gabor
for (i = 1 : nimage)
    Asym{i} = zeros(size(I{1}));  
end
tss = zeros(nimage, 1);  % total scores for testing images
gain = zeros(Totalsketch, 1);  % coding gain for selected Gabor


binnum = 50; % number of bins 
% MEX-C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
mex Clearn.c;   % compile C code
'learning'
tic
Clearn(Totalsketch, stay, ndr+2, alldr, nimage, norient, allfiltered, dr, C, h, sx, sy, T, Mi, Mx, My, Mm, allsymbol(1, :), sym, Asym, Lrange, Orange, sub,  Upperbound, tss, gain, LIK, SHUTUP, binnum);
disp(['mex-C learning time: ' num2str(toc) ' seconds']);

% RESULTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure; 
col = 10; row = floor(nimage/col*2)+1; row = col;
subplot(row, col, 1, 'v6'); imshow(-(sym), []); 
for (i = 1 : nimage)
    subplot(row, col,  1+i*2, 'v6'); 
    imshow(I{i}, []);
    subplot(row, col,  2+i*2, 'v6');
    imshow(-(Asym{i}), []); 
end

figure
gx = 30; gy = 20; 
fr = gray2ind(zeros((sx+gx)*4, (sy+gy)*4, 1, Totalsketch+stay)+1.); 
tx = [0, 1, 1, 2, 2, 3, 3]; 
ty = [2, 0, 2, 0, 2, 0, 2]; 
ttx = [0, 1, 1, 2, 2, 3, 3]; 
tty = [3, 1, 3, 1, 3, 1, 3]; 
for (e = 1:ndr)
    J = I{alldr(e)}; 
    mx = max(J(:)); 
    mn = min(J(:)); 
    JJ{e} =  ((J-mn)/(mx-mn)); 
end
for (i=1:Totalsketch+stay)
[fr(1:sx, 1:sy, 1, i), map] = gray2ind((sqrt(Upperbound+1.)-dr{1, i})/(sqrt(Upperbound+1.))); 
[fr(1:sx, (1:sy)+sy+gy, 1, i), map] = gray2ind((sqrt(Upperbound+1.)-dr{2, i})/(sqrt(Upperbound+1.))); 
for (e = 1:ndr)
[fr((1:sx)+tx(e)*(sx+gx), (1:sy)+ty(e)*(sy+gy), 1, i), map] = gray2ind(JJ{e}); 
[fr((1:sx)+ttx(e)*(sx+gx), (1:sy)+tty(e)*(sy+gy), 1, i), map] = gray2ind((sqrt(Upperbound+1.)-dr{e+2, i})/(sqrt(Upperbound+1.))); 
end
end
mov = immovie(fr, map); 
movie(mov, 20); 

%figure
%montage(fr, map); 

